package net.sf.doolin.gui.action;

import net.sf.doolin.gui.action.support.DefaultActionContextDataExtractor;
import net.sf.doolin.gui.window.opener.ViewOpener;
import net.sf.doolin.util.factory.DataFactory;

import org.springframework.beans.factory.annotation.Required;

/**
 * This action opens a dialog.
 * 
 * @author Damien Coraboeuf (x74639)
 * 
 * @param <T>
 *            Type of the data for the dialog
 */
public abstract class DialogAction<T> extends AbstractSimpleGUIAction {

	private DataFactory<T> modelFactory;
	private ViewOpener<T> dialog;

	@Override
	protected void doExecute(ActionContext actionContext) throws Exception {
		// Data
		Object data = getActionContextData(actionContext);
		// Creates the model
		T model = this.modelFactory.create(data);
		// Opens the view
		ActionContext dialogContext = this.dialog
				.openView(actionContext, model);
		// If valid
		if (dialogContext.getValidationSupport().isValid()) {
			onOK(actionContext, model);
		} else {
			onCancelled(actionContext, model);
		}
	}

	/**
	 * Gets the dialog.
	 * 
	 * @return the dialog
	 */
	public ViewOpener<T> getDialog() {
		return this.dialog;
	}

	/**
	 * Gets the data factory.
	 * 
	 * @return the data factory
	 */
	public DataFactory<T> getModelFactory() {
		return this.modelFactory;
	}

	/**
	 * This method is called when the dialog has been cancelled. By default, it
	 * does nothing.
	 * 
	 * @param actionContext
	 *            Action context
	 * @param bean
	 *            Edited bean
	 * @throws Exception
	 *             If an error occurs while cancelling the dialog
	 */
	protected void onCancelled(ActionContext actionContext, T bean)
			throws Exception {
	}

	/**
	 * This method is called when the dialog has been validated.
	 * 
	 * @param actionContext
	 *            Action context
	 * @param bean
	 *            Edited bean
	 * @throws Exception
	 *             If an error occurs while validating the dialog
	 */
	protected abstract void onOK(ActionContext actionContext, T bean)
			throws Exception;

	/**
	 * Sets the dialog.
	 * 
	 * @param dialog
	 *            the new dialog
	 */
	@Required
	public void setDialog(ViewOpener<T> dialog) {
		this.dialog = dialog;
	}

	/**
	 * Sets the data factory.
	 * 
	 * @param dataFactory
	 *            the new data factory
	 */
	@Required
	public void setModelFactory(DataFactory<T> dataFactory) {
		this.modelFactory = dataFactory;
	}

	/**
	 * Sets if the bean for the dialog is the contextual data of the action
	 * context or its data itself. It is set to <code>false</code> by default,
	 * meaning that the context data (and not the context contextual data) is
	 * used.
	 * 
	 * @param useContext
	 *            <code>true</code> if the bean for the dialog is the contextual
	 *            data of the action context or its data itself.
	 * @deprecated Use
	 *             {@link AbstractGUIAction#setActionContextDataExtractor(net.sf.doolin.gui.action.support.ActionContextDataExtractor)}
	 */
	@Deprecated
	public void setUseContext(boolean useContext) {
		setActionContextDataExtractor(new DefaultActionContextDataExtractor(
				useContext));
	}

}
