package net.sf.doolin.gui.action.support;

import java.awt.AWTKeyStroke;

import javax.swing.AbstractAction;
import javax.swing.Icon;

import net.sf.doolin.bus.support.PropertyChangeSupport;
import net.sf.doolin.gui.display.DisplayState;
import net.sf.doolin.gui.swing.MnemonicInfo;

import org.apache.commons.lang.StringUtils;

/**
 * Ancestor for Swing {@link javax.swing.Action}s used in the Doolin GUI
 * framework.
 * 
 * @author Damien Coraboeuf
 * 
 */
public abstract class AbstractSwingAction extends AbstractAction {

	private static final long serialVersionUID = 1L;

	/**
	 * Property name for the visibility state of the action.
	 */
	public static final String VISIBLE_PROPERTY = "visible";

	private boolean visible = true;

	/**
	 * Checks if the action is visible.
	 * 
	 * @return <code>true</code> if the action is visible
	 */
	public boolean isVisible() {
		return this.visible;
	}

	/**
	 * Sets the icon for this action
	 * 
	 * @param icon
	 *            Icon to set
	 */
	public void setIcon(Icon icon) {
		putValue(SMALL_ICON, icon);
	}

	/**
	 * Sets the shortcut for this action
	 * 
	 * @param shortcut
	 *            Encoded form of the shortcut
	 * @see AWTKeyStroke#getAWTKeyStroke(String)
	 */
	public void setShortcut(String shortcut) {
		if (StringUtils.isBlank(shortcut)) {
			putValue(ACCELERATOR_KEY, null);
		} else {
			putValue(ACCELERATOR_KEY, AWTKeyStroke.getAWTKeyStroke(shortcut));
		}
	}

	/**
	 * Sets the state for the action
	 * 
	 * @param state
	 *            State to set
	 * @see #setEnabled(boolean)
	 * @see #setVisible(boolean)
	 */
	public void setState(DisplayState state) {
		switch (state) {
		case ENABLED:
			setEnabled(true);
			setVisible(true);
			break;
		case DISABLED:
			setEnabled(false);
			setVisible(true);
			break;
		case HIDDEN:
			setEnabled(false);
			setVisible(false);
			break;
		}
	}

	/**
	 * Sets the text of the action, taking case of any declared mnemonic.
	 * 
	 * @param text
	 *            Text of the action
	 * @see MnemonicInfo#configureAction(javax.swing.Action)
	 */
	public void setText(String text) {
		MnemonicInfo info = new MnemonicInfo(text);
		info.configureAction(this);
	}

	/**
	 * Sets the tip for this action
	 * 
	 * @param tip
	 *            Tip
	 */
	public void setTip(String tip) {
		putValue(SHORT_DESCRIPTION, tip);
	}

	/**
	 * Sets the visibility of the action. This property is bound to
	 * {@link #VISIBLE_PROPERTY property}.
	 * 
	 * @param visible
	 *            New visibility of the action
	 */
	public void setVisible(boolean visible) {
		boolean old = this.visible;
		firePropertyChange(VISIBLE_PROPERTY, this.visible,
				this.visible = visible);
		PropertyChangeSupport.change(this, VISIBLE_PROPERTY, old, this.visible);
	}

}
