package net.sf.doolin.gui.action.support;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * This action set a property on the action context data and delegates to an
 * {@link GUIAction action}. If the property path is not set, the action context
 * is not updated.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <T>
 *            Type of the parameter
 */
public class ActionProxyDelegate<T> implements ActionDelegate<T> {

	private boolean useContext;
	private String propertyPath;

	private GUIAction action;

	@Override
	public void call(ActionContext actionContext, T parameter) {
		// Sets the property path if needed
		if (StringUtils.isNotBlank(this.propertyPath)) {
			Object target;
			if (this.useContext) {
				target = actionContext.getContext();
			} else {
				target = actionContext.getData();
			}
			Utils.setProperty(target, this.propertyPath, parameter);
		}
		// Executes the action
		this.action.execute(actionContext);
	}

	/**
	 * Returns the action to execute
	 * 
	 * @return Action
	 */
	public GUIAction getAction() {
		return this.action;
	}

	/**
	 * Returns the path to the property to set
	 * 
	 * @return Property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	/**
	 * Checks if the data or the context must be used
	 * 
	 * @return <code>true</code> if the contextual data must be used
	 */
	public boolean isUseContext() {
		return this.useContext;
	}

	/**
	 * Sets the action to execute
	 * 
	 * @param action
	 *            Action
	 */
	@Required
	public void setAction(GUIAction action) {
		this.action = action;
	}

	/**
	 * Sets the path to the property to set in the action context
	 * 
	 * @param propertyPath
	 *            Property path
	 */
	public void setPropertyPath(String propertyPath) {
		this.propertyPath = propertyPath;
	}

	/**
	 * Sets if the data or the contextual data must be used
	 * 
	 * @param useContext
	 *            <code>true</code> if the contextual data must be used
	 */
	public void setUseContext(boolean useContext) {
		this.useContext = useContext;
	}

}
