package net.sf.doolin.gui.action.task;

import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.service.InfoService;

/**
 * An abstract implementation for {@link GUITask} that provides a default
 * support for the management of the exceptions.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of bean sent to the task
 * @param <R>
 *            Type of bean returned by the task
 */
public abstract class AbstractGUITask<B, R> implements GUITask<B, R> {

	private boolean delegateException;

	/**
	 * If <code>delegateException</code> property is <code>true</code>, a
	 * {@link RuntimeException} is thrown. Otherwise, the
	 * <code>actionContext</code> displays the error.
	 * 
	 * @see InfoService#error(ActionContext, Throwable, String, Object...)
	 */
	@Override
	public void doOnException(ActionContext actionContext, B bean, Throwable th) {
		if (this.delegateException) {
			throw new RuntimeException(th);
		} else {
			actionContext.getApplication().getInfoService().error(
					actionContext, th, "GUITask.error");
		}
	}

	/**
	 * Checks if exception must be delegated to the caller.
	 * 
	 * @return <code>true</code> if exception must be delegated to the caller.
	 */
	public boolean isDelegateException() {
		return this.delegateException;
	}

	/**
	 * Sets if exception must be delegated to the caller.
	 * 
	 * @param delegateException
	 *            <code>true</code> if exception must be delegated to the
	 *            caller.
	 */
	public void setDelegateException(boolean delegateException) {
		this.delegateException = delegateException;
	}

}
