/*
 * Created on Aug 13, 2007
 */
package net.sf.doolin.gui.action.util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.StringReader;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import net.sf.doolin.gui.action.AbstractSimpleGUIAction;
import net.sf.doolin.gui.action.ActionContext;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.service.DesktopService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.swing.SwingUtils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Required;

/**
 * Standard action for displaying an "About..." dialog box.
 * 
 * <p>
 * The displayed dialog's title is set according to the key defined by the
 * <code>title</code> property.
 * 
 * <p>
 * The displayed dialog's text is set according to the key defined by the
 * <code>text</code> property. If <code>html</code> is set to <code>true</code>,
 * the text is rendered as HTML and any link is made active.
 * 
 * <p>
 * An image is put in the dialog is <code>logo</code> is not <code>null</code>.
 * The image is looked using the
 * <code>{@link net.sf.doolin.gui.service.IconService}</code> and the requested
 * size is <code>{@link net.sf.doolin.gui.service.IconSize#BIG}</code>.
 * 
 * <p>
 * The position of the logo relatively to the text is set using the
 * <code>layout</code> property.
 * 
 * @author Damien Coraboeuf
 */
public class ActionAbout extends AbstractSimpleGUIAction {

	private String title;

	private String text;

	private String logo;

	private ImagePosition layout = ImagePosition.LEFT;

	private boolean html;

	/**
	 * Displayed dialog
	 */
	protected JDialog dialog;

	/**
	 * Gets the layout.
	 * 
	 * @return the layout
	 */
	public ImagePosition getLayout() {
		return this.layout;
	}

	/**
	 * Gets the logo.
	 * 
	 * @return the logo
	 */
	public String getLogo() {
		return this.logo;
	}

	/**
	 * Gets the text.
	 * 
	 * @return the text
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getTitle() {
		return this.title;
	}

	/**
	 * Checks if is html.
	 * 
	 * @return true, if is html
	 */
	public boolean isHtml() {
		return this.html;
	}

	/**
	 * Defines if the content is HTML"
	 * 
	 * @param html
	 *            the new html
	 */
	public void setHtml(boolean html) {
		this.html = html;
	}

	/**
	 * Position of the image
	 * 
	 * @param layout
	 *            the new layout
	 */
	public void setLayout(ImagePosition layout) {
		this.layout = layout;
	}

	/**
	 * ID for the logo image
	 * 
	 * @param logo
	 *            the new logo
	 */
	public void setLogo(String logo) {
		this.logo = logo;
	}

	/**
	 * Expression for the text
	 * 
	 * @param text
	 *            the new text
	 */
	@Required
	public void setText(String text) {
		this.text = text;
	}

	@Override
	@Required
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Creates an HTML container for the text.
	 * 
	 * @param context
	 *            Action context
	 * @param dialogText
	 *            Text to display
	 * 
	 * @return Component to be displayed for the text
	 */
	protected Component createHTML(final ActionContext context,
			String dialogText) {
		HTMLEditorKit editorKit = new HTMLEditorKit();
		final JEditorPane editorPane = new JEditorPane();
		editorPane.setEditable(false);
		editorPane.setEditorKit(editorKit);
		// Listens for the links
		editorPane.addHyperlinkListener(new HyperlinkListener() {

			public void hyperlinkUpdate(HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
					URL url = e.getURL();
					context.getApplication().getBean(DesktopService.class)
							.browse(url);
				}
			}

		});
		// HTML content
		try {
			// Get a reader on the HTML
			BufferedReader reader = new BufferedReader(new StringReader(
					dialogText));
			try {
				// Creates the document
				HTMLDocument document = (HTMLDocument) editorKit
						.createDefaultDocument();
				// Reads the document
				editorKit.read(reader, document, 0);
				// Ok
				editorPane.setDocument(document);
			} finally {
				reader.close();
			}
		} catch (Exception ex) {
			throw new RuntimeException("Cannot parse About text as HTML", ex);
		}
		// Ok
		return editorPane;
	}

	@Override
	protected void doExecute(ActionContext context) throws Exception {
		Object data = context.getData();
		// Title
		String dialogTitle = new GUIExpression(
				context.getSubscriberValidator(), data, this.title).getValue();
		// Text
		String dialogText = new GUIExpression(context.getSubscriberValidator(),
				data, this.text).getValue();
		// Text panel
		JPanel panelText = new JPanel(new BorderLayout());
		panelText.setBackground(Color.WHITE);
		panelText.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		panelText.add(createText(context, dialogText), BorderLayout.CENTER);
		// OK button
		JPanel panelOk = new JPanel(new FlowLayout(4, 4, FlowLayout.TRAILING));
		panelOk.setBackground(Color.WHITE);
		JButton buttonOk = new JButton(UIManager
				.getString("OptionPane.okButtonText"));
		panelOk.add(buttonOk);
		// Content & buttons
		JPanel panelContent = new JPanel(new BorderLayout());
		panelContent.add(panelText, BorderLayout.CENTER);
		panelContent.add(panelOk, BorderLayout.PAGE_END);
		// Main panel
		JPanel panelMain = new JPanel(new BorderLayout());
		panelMain.setBackground(Color.WHITE);
		panelMain.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		panelMain.add(panelContent, BorderLayout.CENTER);
		// Logo image
		if (StringUtils.isNotBlank(this.logo)) {
			Icon logoImage = getIconService().getIcon(this.logo, IconSize.BIG);
			Object constraint;
			if (this.layout == ImagePosition.TOP) {
				constraint = BorderLayout.PAGE_START;
			} else {
				constraint = BorderLayout.LINE_START;
			}
			JLabel labelLogo = new JLabel(logoImage);
			labelLogo.setOpaque(false);
			labelLogo.setVerticalAlignment(SwingConstants.TOP);
			panelMain.add(labelLogo, constraint);
		}
		// Dialog
		this.dialog = SwingUtils.createWindow(JDialog.class, context
				.getComponent());
		this.dialog.setModal(true);
		this.dialog.setUndecorated(true);
		this.dialog.setTitle(dialogTitle);
		this.dialog.setContentPane(panelMain);
		this.dialog.pack();
		this.dialog.setLocationRelativeTo(context.getComponent());
		// OK button
		this.dialog.getRootPane().setDefaultButton(buttonOk);
		buttonOk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ActionAbout.this.dialog.dispose();
			}
		});
		// Displays
		this.dialog.setVisible(true);
	}

	/**
	 * Creates the text component
	 * 
	 * @param context
	 *            Action context
	 * @param dialogText
	 *            Text to display
	 * @return Component to be displayed for the text
	 */
	private Component createText(ActionContext context, String dialogText) {
		if (this.html) {
			return createHTML(context, dialogText);
		} else {
			return new JLabel(dialogText);
		}
	}

	/**
	 * Positions for the image.
	 * 
	 * @author Damien Coraboeuf
	 */
	public static enum ImagePosition {

		/**
		 * Positions the image at the top of the dialog
		 */
		TOP,

		/**
		 * Positions the image at the left of the dialog
		 */
		LEFT;

	}

}
