package net.sf.doolin.gui.bus;

import java.lang.ref.WeakReference;

import net.sf.doolin.bus.Bus;
import net.sf.doolin.bus.Subscriber;
import net.sf.doolin.bus.SubscriberInfo;

import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;

/**
 * Bean model for the representation of a {@link Bus} subscription.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class BusMonitoringSubscription {

	private final int id;
	private final String trigger;
	private final String execution;
	private final String validator;
	private final String type;

	private final WeakReference<Subscriber<?>> subscriber;
	private final DateTime registrationTimestamp;
	private final long registrationPeriod;

	/**
	 * Constructor
	 * 
	 * @param id
	 *            ID for this subscription
	 * @param subscriberInfo
	 *            Associated subscriber information
	 */
	public BusMonitoringSubscription(int id, SubscriberInfo subscriberInfo) {
		this.id = id;
		Subscriber<?> sub = subscriberInfo.getSubscriber();
		this.type = getType(sub);
		this.trigger = sub.getTriggerDescription();
		this.execution = sub.getExecutionDescription();
		this.validator = sub.getValidatorDescription();
		// Registration timestamp
		long registrationTime = subscriberInfo.getRegistrationTime();
		this.registrationTimestamp = new DateTime(registrationTime);
		// Registration duration
		this.registrationPeriod = (System.currentTimeMillis() - registrationTime);
		// Subscriber reference
		this.subscriber = new WeakReference<Subscriber<?>>(sub);
	}

	public String getExecution() {
		return this.execution;
	}

	/**
	 * Returns the ID of this subscription
	 * 
	 * @return ID of this subscription
	 */
	public int getId() {
		return this.id;
	}

	public long getRegistrationPeriod() {
		return this.registrationPeriod;
	}

	public DateTime getRegistrationTimestamp() {
		return this.registrationTimestamp;
	}

	public String getTrigger() {
		return this.trigger;
	}

	/**
	 * Returns the type of this subscription
	 * 
	 * @return Type of this subscription
	 */
	public String getType() {
		return this.type;
	}

	private String getType(Subscriber<?> subscriber) {
		String type = subscriber.getClass().getName();
		type = StringUtils.substringBefore(StringUtils.substringAfterLast(type,
				"."), "$");
		return type;
	}

	public String getValidator() {
		return this.validator;
	}

	/**
	 * Tests if the associated subscription is still valid.
	 * 
	 * @return <code>true</code> if the subscriber is still available and still
	 *         {@linkplain Subscriber#isValid() valid}.
	 */
	public boolean test() {
		Subscriber<?> s = this.subscriber.get();
		if (s != null) {
			return s.isValid();
		} else {
			return false;
		}
	}

	@Override
	public String toString() {
		return String.format("%s -> %s (%s)", this.trigger, this.execution,
				this.validator);
	}

}
