package net.sf.doolin.gui.display;

import java.util.List;

import net.sf.doolin.gui.action.ActionContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Display state handler that operates a conjunction between several handlers.
 * 
 * Conjuction rules are defined as follow:
 * <ul>
 * <li>At least one handler returns <code>HIDDEN</code>, result is
 * <code>HIDDEN</code>
 * <li>At least one handler returns <code>DISABLED</code>, result is
 * <code>DISABLED</code>
 * <li>All handlers return <code>ENABLED</code>, result is <code>ENABLED</code>
 * </ul>
 * 
 * @author Damien Coraboeuf
 * 
 */
public class AndDisplayStateHandler extends MultiDisplayStateHandler {

	private static final Logger log = LoggerFactory
			.getLogger(AndDisplayStateHandler.class);

	/**
	 * Default constructor
	 */
	public AndDisplayStateHandler() {
	}

	/**
	 * Constructor from a list.
	 * 
	 * @param handlers
	 *            List of handlers
	 */
	public AndDisplayStateHandler(List<DisplayStateHandler> handlers) {
		super(handlers);
	}

	/**
	 * Constructor from an array
	 * 
	 * @param handlers
	 *            Array of handlers
	 */
	public AndDisplayStateHandler(DisplayStateHandler... handlers) {
		super(handlers);
	}

	@Override
	public DisplayState getDisplayState(ActionContext actionContext) {
		log.debug(String.format("Computing state change for context %s",
				actionContext.getContext()));
		boolean atLeastOneDisabled = false;
		boolean atLeastOneHidden = false;
		for (DisplayStateHandler handler : this.getHandlers()) {
			DisplayState state = handler.getDisplayState(actionContext);
			log.debug(String.format("Sub-model returning %s", state));
			switch (state) {
				case ENABLED :
					break;
				case DISABLED :
					atLeastOneDisabled = true;
					break;
				case HIDDEN :
					atLeastOneHidden = true;
					break;
			}
		}
		if (atLeastOneHidden) {
			return DisplayState.HIDDEN;
		} else if (atLeastOneDisabled) {
			return DisplayState.DISABLED;
		} else {
			return DisplayState.ENABLED;
		}
	}

}
