package net.sf.doolin.gui.field;

import javax.swing.JPasswordField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.util.PasswordUtils;

import com.google.common.base.Function;
import com.google.common.base.Functions;

/**
 * This field is used to enter a password.
 * <p>
 * The target property that hosts the password value in the model can be either
 * a character array (<code>char[]</code>) or the result of a digest (as
 * returned by the {@link PasswordUtils#digestPassword(char[])} method. This
 * field mode is chosen according to the value of the
 * {@link #setDigest(boolean) digest} property.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of bean for the view
 */
public class FieldPassword<V> extends AbstractPropertyFieldDescriptor<V> {

	private static final long serialVersionUID = 1L;

	private int columns = 8;
	private FieldPasswordStore store;

	protected Function<char[], ? extends Object> createConverter() {
		switch (this.store) {
		case CHAR:
			return Functions.identity();
		case DIGEST:
			return new Function<char[], String>() {

				@Override
				public String apply(char[] source) {
					return PasswordUtils.digestPassword(source);
				}
			};
		case PLAIN:
			return new Function<char[], String>() {

				@Override
				public String apply(char[] source) {
					return new String(source);
				}
			};
		default:
			throw new IllegalStateException(String.format(
					"Password store type %s is not managed", this.store));
		}
	}

	@Override
	public Field<V> createField(final GUIView<V> view) {
		final JPasswordField field = new JPasswordField(this.columns);
		// Storage adapter
		final Function<char[], ? extends Object> converter = createConverter();
		// Conversion listener
		DocumentListener listener = new DocumentListener() {

			@Override
			public void changedUpdate(DocumentEvent e) {
				updateModel();
			}

			@Override
			public void insertUpdate(DocumentEvent e) {
				updateModel();
			}

			@Override
			public void removeUpdate(DocumentEvent e) {
				updateModel();
			}

			private void updateModel() {
				char[] password = field.getPassword();
				Object value = converter.apply(password);
				setProperty(view, value);
			}

		};
		// Sets the adapter
		field.getDocument().addDocumentListener(listener);
		// OK
		return new SimpleField<V>(view, this, field);
	}

	/**
	 * Gets the number of columns for the password field
	 * 
	 * @return Number of columns
	 */
	public int getColumns() {
		return this.columns;
	}

	public FieldPasswordStore getStore() {
		return this.store;
	}

	/**
	 * Checks if the field value must be renderer as a
	 * {@link PasswordUtils#digestPassword(char[]) digest} (<code>true</code>)
	 * or a character array (<code>false</code>).
	 * 
	 * @return Digest mode
	 * @deprecated Use {@link #getStore()} instead
	 */
	@Deprecated
	public boolean isDigest() {
		return FieldPasswordStore.DIGEST == this.store;
	}

	/**
	 * Sets the number of columns for the password field
	 * 
	 * @param columns
	 *            Number of columns
	 * @see JPasswordField#setColumns(int)
	 */
	public void setColumns(int columns) {
		this.columns = columns;
	}

	/**
	 * Sets if the field value must be renderer as a
	 * {@link PasswordUtils#digestPassword(char[]) digest} (<code>true</code>)
	 * or a character array (<code>false</code>).
	 * 
	 * @param digest
	 *            Digest mode
	 * @deprecated Use {@link #setStore(FieldPasswordStore)} instead
	 */
	@Deprecated
	public void setDigest(boolean digest) {
		if (digest) {
			setStore(FieldPasswordStore.DIGEST);
		} else {
			setStore(FieldPasswordStore.CHAR);
		}
	}

	public void setStore(FieldPasswordStore store) {
		this.store = store;
	}

}
