package net.sf.doolin.gui.field.list;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.Enumeration;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;

import net.sf.doolin.gui.display.DisplayState;
import net.sf.doolin.gui.field.Field;
import net.sf.doolin.gui.field.support.SimpleField;
import net.sf.doolin.gui.swing.LabelInfo;
import net.sf.doolin.gui.swing.MnemonicInfo;
import net.sf.doolin.gui.util.PropertyContainer;
import net.sf.doolin.gui.view.GUIView;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;

import ca.odell.glazedlists.EventList;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * Field descriptor for a group of radio buttons.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <V>
 *            Type of object for the view
 * @param <E>
 *            Type of object in the item list
 */
public class FieldRadioGroup<V, E> extends
		AbstractListPropertyFieldDescriptor<V, E> {

	/**
	 * Field property for the button group
	 * 
	 * @see PropertyContainer#getProperty(Object)
	 */
	public static final String BUTTON_GROUP = "ButtonGroup";

	private static final long serialVersionUID = 1L;

	private String space = "0dlu";
	private int orientation = SwingConstants.VERTICAL;

	/**
	 * Creates a radio button for a given value.
	 * 
	 * @param view
	 *            Hosting view
	 * @param field
	 *            Field for the group
	 * @param item
	 *            Item to associate the radio button with
	 * @return Radio button
	 */
	protected JRadioButton buildRadio(final GUIView<V> view,
			SimpleField<V> field, final E item) {
		// Label info
		LabelInfo info = getLabelInfoProvider().getLabelIcon(item);
		String text = info.getText();
		MnemonicInfo mnemonicInfo = new MnemonicInfo(text);
		// Radio button
		final JRadioButton button = new JRadioButton();
		mnemonicInfo.configureButton(button);
		// Button group
		ButtonGroup buttonGroup = (ButtonGroup) field.getProperty(BUTTON_GROUP);
		buttonGroup.add(button);
		// Sync. flag
		final AtomicBoolean onUpdate = new AtomicBoolean(false);
		// Model -> Button
		Runnable updateButtonTask = new Runnable() {

			@Override
			public void run() {
				if (!onUpdate.get()) {
					onUpdate.set(true);
					try {
						@SuppressWarnings("unchecked")
						E value = (E) getProperty(view);
						boolean selected = ObjectUtils.equals(item, value);
						button.setSelected(selected);
					} finally {
						onUpdate.set(false);
					}
				}
			}
		};
		subscribe(view, updateButtonTask);
		// Button -> Model
		button.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (!onUpdate.get()) {
					onUpdate.set(true);
					try {
						setProperty(view, item);
					} finally {
						onUpdate.set(false);
					}
				}
			}
		});
		// Initial value
		updateButtonTask.run();
		// OK
		return button;
	}

	/**
	 * Creates the radio button list
	 * 
	 * @param view
	 *            Hosting view
	 * @param field
	 *            Field for the group
	 * @param container
	 *            Panel that must contain the radio buttons
	 */
	protected void buildRadioList(GUIView<V> view, SimpleField<V> field,
			JPanel container) {
		// Removes all components
		container.removeAll();
		// Gets the source list
		EventList<E> list = getListModel(view);
		// Layout (vertical by default)
		String rowSpecs = StringUtils.join(Collections.nCopies(list.size(),
				"default"), "," + this.space + ",");
		String columnSpecs = "fill:default:grow";
		if (this.orientation == SwingConstants.HORIZONTAL) {
			// Switches layout
			String temp = columnSpecs;
			columnSpecs = rowSpecs;
			rowSpecs = temp;
		}
		FormLayout layout = new FormLayout(columnSpecs, rowSpecs);
		container.setLayout(layout);
		// Radio group
		ButtonGroup buttonGroup = new ButtonGroup();
		field.setProperty(BUTTON_GROUP, buttonGroup);
		// For each item
		int index = 1;
		CellConstraints cc = new CellConstraints();
		for (E item : list) {
			JRadioButton radio = buildRadio(view, field, item);
			CellConstraints constraint;
			if (this.orientation == SwingConstants.VERTICAL) {
				constraint = cc.xy(1, index * 2 - 1);
			} else {
				constraint = cc.xy(index * 2 - 1, 1);
			}
			container.add(radio, constraint);
			index++;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Field<V> createField(final GUIView<V> view) {
		// Container
		JPanel container = new JPanel();
		// Field
		SimpleField<V> field = new SimpleField<V>(view, this, container);
		// Builds the radio list
		buildRadioList(view, field, container);
		// OK
		return field;
	}

	public int getOrientation() {
		return this.orientation;
	}

	/**
	 * Gets the layout space between the buttons.
	 * 
	 * @return Layout space
	 */
	public String getSpace() {
		return this.space;
	}

	@Override
	public void setDisplayState(Field<V> field, JComponent component,
			DisplayState displayState) {
		// Gets the button group
		ButtonGroup buttonGroup = (ButtonGroup) field.getProperty(BUTTON_GROUP);
		// For each button
		Enumeration<AbstractButton> buttons = buttonGroup.getElements();
		while (buttons.hasMoreElements()) {
			AbstractButton button = buttons.nextElement();
			switch (displayState) {
			case ENABLED:
				button.setEnabled(true);
				button.setVisible(true);
				break;
			case DISABLED:
				button.setEnabled(false);
				button.setVisible(true);
				break;
			case HIDDEN:
				button.setEnabled(false);
				button.setVisible(false);
				break;
			}
		}
		// Default
		super.setDisplayState(field, component, displayState);
	}

	/**
	 * Sets the orientation of the buttons in the group. It is either
	 * {@link SwingConstants#VERTICAL} (the default) or
	 * {@link SwingConstants#HORIZONTAL}.
	 */
	public void setOrientation(int orientation) {
		if (orientation == SwingConstants.VERTICAL
				|| orientation == SwingConstants.HORIZONTAL) {
			this.orientation = orientation;
		} else {
			throw new IllegalArgumentException(
					String
							.format(
									"The orientation must be either %d (SwingConstants.VERTICAL) or %d (SwingConstants.HORIZONTAL)",
									SwingConstants.VERTICAL,
									SwingConstants.HORIZONTAL));
		}
	}

	/**
	 * Sets the layout space between the buttons. It is set by default to
	 * "0dlu".
	 * 
	 * @param space
	 *            Layout space
	 */
	public void setSpace(String space) {
		this.space = space;
	}
}
