package net.sf.doolin.gui.spring;

import java.awt.Color;
import java.awt.SystemColor;
import java.beans.PropertyEditorSupport;

import net.sf.doolin.util.Utils;

import org.apache.commons.lang.StringUtils;

/**
 * Editor for the {@link Color} type.
 * <p>
 * If the text is blank, the returned color is {@link Color#black}.
 * <p>
 * If the text starts with #, the text is considered as an HEX string of 3
 * bytes, for red, green and blue. For example, #FFFF00 stands for yellow.
 * <p>
 * In any other case, the text is considered as a constant in either
 * {@link SystemColor} or {@link Color}.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ColorEditor extends PropertyEditorSupport {

	@Override
	public void setAsText(String text) throws IllegalArgumentException {
		Color color = parseColor(text);
		setValue(color);
	}

	/**
	 * Parses the text to a color
	 * 
	 * @param text
	 *            Text to parse
	 * @return Parser color
	 * @throws IllegalArgumentException
	 *             If the text cannot be parsed
	 */
	public static Color parseColor(String text) throws IllegalArgumentException {
		if (StringUtils.isBlank(text)) {
			return Color.BLACK;
		} else if (text.startsWith("#")) {
			String hex = text.substring(1);
			if (hex.length() != 6) {
				throw new IllegalArgumentException("Cannot parse color " + text);
			} else {
				try {
					String redValue = hex.substring(0, 2);
					String greenValue = hex.substring(2, 4);
					String blueValue = hex.substring(4, 6);
					int red = parseHex(redValue);
					int green = parseHex(greenValue);
					int blue = parseHex(blueValue);
					return new Color(red, green, blue);
				} catch (Exception ex) {
					throw new IllegalArgumentException("Cannot parse color "
							+ text);
				}
			}
		} else if (text.startsWith("@")) {
			String name = text.substring(1);
			return (Color) Utils.getConstant(SystemColor.class, name);
		} else {
			return (Color) Utils.getConstant(Color.class, text);
		}
	}

	/**
	 * Returns the Hex representation of the color
	 * 
	 * @param c
	 *            Color
	 * @return 0xRRGGBB
	 */
	public static String toString(Color c) {
		if (c == null) {
			return "";
		} else {
			int r = c.getRed();
			int g = c.getGreen();
			int b = c.getBlue();
			return String.format("#%02X%02X%02X", r, g, b);
		}
	}

	private static int parseHex(String value) {
		return Integer.parseInt(value, 16);
	}

}
