package net.sf.doolin.gui.spring;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import net.sf.doolin.context.spring.CustomNamespaces;
import net.sf.doolin.context.spring.SimpleBeanFactory;
import net.sf.doolin.context.spring.SimpleDefinitionParser;
import net.sf.doolin.gui.action.path.list.ParsedActionPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.action.support.DefaultActionContextDataExtractor;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.app.support.GUIWindowStartup;
import net.sf.doolin.gui.display.AdaptedContextPropertyDisplayStateHandler;
import net.sf.doolin.gui.display.AlwaysDisabledDisplayStateHandler;
import net.sf.doolin.gui.display.AlwaysEnabledDisplayStateHandler;
import net.sf.doolin.gui.display.AlwaysHiddenDisplayStateHandler;
import net.sf.doolin.gui.display.AndDisplayStateHandler;
import net.sf.doolin.gui.display.BooleanStateAdapter;
import net.sf.doolin.gui.display.ConditionalDisplayStateHandler;
import net.sf.doolin.gui.display.ConditionalJEXLDisplayStateHandler;
import net.sf.doolin.gui.display.ConditionalStateAdapter;
import net.sf.doolin.gui.display.ContextDisplayStateHandler;
import net.sf.doolin.gui.display.MapStateAdapter;
import net.sf.doolin.gui.display.ValidationDisplayStateHandler;
import net.sf.doolin.gui.docking.DockingConstraint;
import net.sf.doolin.gui.form.FormViewDescriptor;
import net.sf.doolin.gui.layout.FormLayoutService;
import net.sf.doolin.gui.layout.TableLayoutService;
import net.sf.doolin.gui.model.PropertyDataFactory;
import net.sf.doolin.gui.spring.table.ColumnDateDecorator;
import net.sf.doolin.gui.spring.table.ColumnEnumDecorator;
import net.sf.doolin.gui.spring.table.ColumnExpressionDecorator;
import net.sf.doolin.gui.spring.table.ColumnParser;
import net.sf.doolin.gui.spring.table.ColumnsParser;
import net.sf.doolin.gui.spring.table.TableParser;
import net.sf.doolin.gui.spring.tree.TreeChildrenParser;
import net.sf.doolin.gui.spring.tree.TreeNodeParser;
import net.sf.doolin.gui.spring.tree.TreeParser;
import net.sf.doolin.gui.util.GUILF;
import net.sf.doolin.gui.validation.SimpleValidator;
import net.sf.doolin.gui.view.decorator.DecorationStyle;
import net.sf.doolin.gui.window.descriptor.ViewReference;
import net.sf.doolin.gui.window.dialog.NoButtonDialogActionsFactory;
import net.sf.doolin.gui.window.dialog.OKOnlyDialogActionsFactory;
import net.sf.doolin.gui.window.opener.InnerViewOpener;
import net.sf.doolin.gui.window.support.DialogWindowFactory;
import net.sf.doolin.gui.window.support.WindowSize;
import net.sf.doolin.util.Utils;
import net.sf.doolin.util.factory.SimpleDataFactory;

import org.springframework.beans.factory.xml.NamespaceHandlerSupport;

/**
 * Handler for the GUI schema. See the <a
 * href="http://doolin-guif.net/wiki/doku.php?id=gui:custom_schema">Wiki</a> for
 * the detail of elements.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class GUINamespaceHandler extends NamespaceHandlerSupport {

	private SimpleDefinitionParser<ParsedActionPathList> createActionPathListParser() {
		return SimpleDefinitionParser.getInstance(ParsedActionPathList.class)
				.setListString("actionPath", "paths").setContainerProperty(
						"actionPathList");
	}

	@Override
	public void init() {
		CustomNamespaces.addCustomNamespace(GUI.NS);

		/*
		 * DECORATORS
		 */

		// gui:properties
		registerBeanDefinitionDecorator("properties", new PropertiesDecorator());

		// Beans

		/*
		 * SERVICES
		 */

		// gui:propertyIconService
		registerBeanDefinitionParser("propertyIconService",
				new PropertyIconServiceParser());

		/*
		 * MISC
		 */

		// gui:lookAndFeel
		registerBeanDefinitionParser("lookAndFeel", SimpleDefinitionParser
				.getInstance(GUILF.class).addSimpleAttribute("useSystem")
				.addSimpleAttribute("lfClassName").setDefaultID("LookAndFeel"));
		// gui:properties
		registerBeanDefinitionParser("properties", new PropertiesDecorator());

		/*
		 * START-UP
		 */

		// gui:application
		SimpleDefinitionParser<GUIApplication> applicationParser = SimpleDefinitionParser
				.getInstance(GUIApplication.class).setDefaultID("Application");
		registerBeanDefinitionParser("application", applicationParser);
		// gui:startup-window
		registerSimpleParser("startup-window", SimpleDefinitionParser
				.getInstance(GUIWindowStartup.class).addAttributeRef(
						"windowDescriptorRef", "windowDescriptor")
				.addAttributeRef("dataFactoryRef", "dataFactory")
				.setContainerProperty("startup").addFactoryFromAttribute(
						"beanClass", "dataFactory", new SimpleBeanFactory() {

							@Override
							public Object create(String value) {
								@SuppressWarnings("unchecked")
								Class<Object> beanClass = (Class<Object>) Utils
										.forClass(value);
								SimpleDataFactory<?> factory = new SimpleDataFactory<Object>(
										beanClass);
								return factory;
							}

						}));

		/*
		 * ACTION
		 */

		// gui:actionPathList
		registerBeanDefinitionParser("actionPathList",
				createActionPathListParser());

		// Menu bars
		registerSimpleParser("menuBar", createActionPathListParser()
				.setContainerProperty("menuBar"));
		registerSimpleParser("windowMenuBar", createActionPathListParser()
				.setContainerProperty("windowMenuBar"));
		registerSimpleParser("tableMenu", createActionPathListParser()
				.setContainerProperty("tableMenu"));

		// Tool bars
		registerSimpleParser("toolBar", new ToolBarDecorator());
		registerSimpleParser("windowToolBar", SimpleDefinitionParser
				.getInstance(ToolBar.class).addSimpleAttribute("name")
				.addSimpleAttribute("position").setContainerProperty(
						"windowToolBar"));
		registerSimpleParser("tableToolBar", createActionPathListParser()
				.setContainerProperty("tableToolBar"));
		registerSimpleParser("viewToolBar", createActionPathListParser()
				.setContainerProperty("viewToolBar"));

		// Action context data extractors

		registerSimpleParser("useContextData", SimpleDefinitionParser
				.getInstance(DefaultActionContextDataExtractor.class)
				.addPropertyValue("useContext", "true").setContainerProperty(
						"actionContextDataExtractor"));

		registerSimpleParser("contextDataExtractor", SimpleDefinitionParser
				.getInstance(DefaultActionContextDataExtractor.class)
				.addSimpleAttribute("useContext").addAttributeRef(
						"dataFactoryRef", "dataFactory").setContainerProperty(
						"actionContextDataExtractor"));

		/*
		 * WINDOW
		 */

		// gui:layoutWindow
		registerBeanDefinitionParser("layoutWindow", new LayoutWindowParser());
		// gui:tabbedWindow
		registerBeanDefinitionParser("tabbedWindow", new TabbedWindowParser());
		// gui:simpleWindow
		registerBeanDefinitionParser("simpleWindow", new SimpleWindowParser());

		// gui:dialog
		registerSimpleParser("dialog", SimpleDefinitionParser.getInstance(
				DialogWindowFactory.class)
				.setContainerProperty("windowFactory"));
		// gui:okOnlyDialog
		registerSimpleParser("okOnlyDialog", SimpleDefinitionParser
				.getInstance(OKOnlyDialogActionsFactory.class)
				.setContainerProperty("dialogActionsFactory"));
		// gui:noButtonDialog
		registerSimpleParser("noButtonDialog", SimpleDefinitionParser
				.getInstance(NoButtonDialogActionsFactory.class)
				.setContainerProperty("dialogActionsFactory"));

		// gui:windowSize
		registerSimpleParser("windowSize", SimpleDefinitionParser.getInstance(
				WindowSize.class).addSimpleAttribute("height")
				.addSimpleAttribute("width").addSimpleAttribute("pack")
				.addSimpleAttribute("centered").addSimpleAttribute("maximized")
				.addSimpleAttribute("saveInPrefs").setContainerProperty("size"));

		// gui:viewReferences
		registerListDecorator("viewReferences", "viewReferences");

		// gui:viewReference
		registerBeanDefinitionParser("viewReference", SimpleDefinitionParser
				.getInstance(ViewReference.class).addAttributeRef(
						"viewDescriptor").addSimpleAttribute("visible")
				.addSimpleAttribute("viewConstraint").addFactoryFromAttribute(
						"propertyPath", "modelFactory",
						new SimpleBeanFactory() {
							@SuppressWarnings("unchecked")
							public Object create(String value) {
								PropertyDataFactory factory = new PropertyDataFactory();
								factory.setPropertyPath(value);
								return factory;
							}
						}));

		// gui:decorationStyle
		registerSimpleParser("decorationStyle", SimpleDefinitionParser
				.getInstance(DecorationStyle.class)
				.setMap(null, "parameterMap").setContainerProperty(
						"decorationStyle"));

		// gui:simpleValidator
		registerSimpleParser("simpleValidator", SimpleDefinitionParser
				.getInstance(SimpleValidator.class).setListString(
						"simpleValidation", "validations").setMap(
						"objectValidators", "objectValidators")
				.setContainerProperty("validator"));

		/*
		 * DOCKING
		 */

		// gui:dockingWindow
		registerBeanDefinitionParser("dockingWindow", new DockingWindowParser());

		// gui:dockingConstraint
		registerSimpleParser("dockingConstraint", SimpleDefinitionParser
				.getInstance(DockingConstraint.class).addSimpleAttribute(
						"externalizable").addSimpleAttribute("zoomable")
				.addSimpleAttribute("closeable").addSimpleAttribute("moveable")
				.addSimpleAttribute("header").addSimpleAttribute("dockingPath")
				.setContainerProperty("viewConstraint"));

		/*
		 * VIEW OPENERS
		 */

		// gui:openView
		registerBeanDefinitionParser("openView", SimpleDefinitionParser
				.getInstance(InnerViewOpener.class).addAttributeRef(
						"viewDescriptor"));

		/*
		 * FORM
		 */

		// gui:formView
		registerSimpleParser("formView", SimpleDefinitionParser.getInstance(
				FormViewDescriptor.class).addSimpleAttribute("viewId")
				.addSimpleAttribute("scrollable").addSimpleAttribute(
						"titleExpression").addSimpleAttribute("iconId")
				.addAttributeRef("validationDecorator").addAttributeRef(
						"validator").addAttributeRef("viewDecorator")
				.addAttributeRef("closeAction").setContainerProperty(
						"viewDescriptor"));

		// gui:formLayout
		registerSimpleParser("formLayout", SimpleDefinitionParser.getInstance(
				FormLayoutService.class).addSimpleAttribute("columnSpecs")
				.addSimpleAttribute("columnGroups").addSimpleAttribute(
						"rowSpecs").addSimpleAttribute("rowSpec")
				.addSimpleAttribute("rowGap").addSimpleAttribute("rowCount")
				.addSimpleAttribute("rowGroups").addSimpleAttribute(
						"defaultBorder").setContainerProperty("layout"));

		// gui:tableLayout
		registerSimpleParser("tableLayout", SimpleDefinitionParser.getInstance(
				TableLayoutService.class).addSimpleAttribute("columnSpecs")
				.setContainerProperty("layout"));

		// gui:borderLayout
		registerBeanDefinitionParser(
				"borderLayout",
				new LayoutManagerServiceParser<BorderLayout>(BorderLayout.class)
						.addAttribute("hgap").addAttribute("vgap"));

		// gui:flowLayout
		registerBeanDefinitionParser("flowLayout",
				new LayoutManagerServiceParser<FlowLayout>(FlowLayout.class)
						.addAttribute("hgap").addAttribute("vgap")
						.addConstantAttribute(FlowLayout.class, "alignment"));

		// gui:fields
		registerListDecorator("fields", "fieldList");

		/*
		 * FIELDS
		 */

		// gui:section
		registerBeanDefinitionParser("section", new SectionParser());
		// gui:separator
		registerBeanDefinitionParser("separator", new SeparatorParser());
		// gui:label
		registerBeanDefinitionParser("label", new LabelParser());
		// gui:label-info
		registerBeanDefinitionParser("label-info", new LabelInfoParser());
		// gui:button
		registerBeanDefinitionParser("button", new ButtonParser());
		// gui:text
		registerBeanDefinitionParser("text", new TextParser());
		// gui:password
		registerBeanDefinitionParser("password", new PasswordParser());
		// gui:memo
		registerBeanDefinitionParser("memo", new MemoParser());
		// gui:integer
		registerBeanDefinitionParser("integer", new IntegerParser());
		// gui:long
		registerBeanDefinitionParser("long", new LongParser());
		// gui:decimal
		registerBeanDefinitionParser("decimal", new DecimalParser());
		// gui:combo
		registerBeanDefinitionParser("combo", new ComboParser());
		// gui:multicombo
		registerBeanDefinitionParser("multicombo", new MultiComboParser());
		// gui:radio-group
		registerBeanDefinitionParser("radio-group", new RadioGroupParser());
		// gui:check
		registerBeanDefinitionParser("check", new CheckParser());
		// gui:radio
		registerBeanDefinitionParser("radio", new RadioParser());
		// gui:toggle
		registerBeanDefinitionParser("toggle", new ToggleParser());
		// gui:color
		registerBeanDefinitionParser("color", new ColorParser());
		// gui:font
		registerBeanDefinitionParser("font", new FontParser());
		// gui:file
		registerBeanDefinitionParser("file", new FileParser());

		/*
		 * DISPLAY STATE HANDLERS
		 */

		initDisplayStateHandlers();

		/*
		 * TABLE
		 */

		// gui:table
		registerBeanDefinitionParser("table", new TableParser());

		// gui:columns
		registerSimpleParser("columns", new ColumnsParser());
		// gui:column
		registerBeanDefinitionParser("column", new ColumnParser());

		// gui:column-date
		registerSimpleParser("column-date", new ColumnDateDecorator());
		// gui:column-enum
		registerSimpleParser("column-enum", new ColumnEnumDecorator());
		// gui:column-expression
		registerSimpleParser("column-expression",
				new ColumnExpressionDecorator());

		/*
		 * TREE
		 */

		// gui:tree
		registerBeanDefinitionParser("tree", new TreeParser());

		// gui:tree-node(s)
		registerListDecorator("tree-nodes", "nodes");
		registerBeanDefinitionParser("tree-node", new TreeNodeParser());
		// gui:tree-children(-list)
		registerListDecorator("tree-children-list", "children");
		registerBeanDefinitionParser("tree-children", new TreeChildrenParser());
	}

	private void initDisplayStateHandlers() {
		// Fixed states
		registerSimpleParser("stateDisabled", SimpleDefinitionParser
				.getInstance(AlwaysDisabledDisplayStateHandler.class)
				.setContainerProperty("displayStateHandler"));
		registerSimpleParser("stateEnabled", SimpleDefinitionParser
				.getInstance(AlwaysEnabledDisplayStateHandler.class)
				.setContainerProperty("displayStateHandler"));
		registerSimpleParser("stateHidden", SimpleDefinitionParser.getInstance(
				AlwaysHiddenDisplayStateHandler.class).setContainerProperty(
				"displayStateHandler"));

		// gui:stateValid
		registerSimpleParser("stateValid", SimpleDefinitionParser.getInstance(
				ValidationDisplayStateHandler.class).addSimpleAttribute(
				"notValidState").setContainerProperty("displayStateHandler"));

		// gui:stateContextNotNull
		registerSimpleParser("stateContextNotNull", SimpleDefinitionParser
				.getInstance(ContextDisplayStateHandler.class)
				.setContainerProperty("displayStateHandler"));

		// gui:stateCondition
		registerSimpleParser("stateCondition", SimpleDefinitionParser
				.getInstance(ConditionalDisplayStateHandler.class)
				.addSimpleAttribute("propertyPath").addSimpleAttribute(
						"operator").addSimpleAttribute("operandPath")
				.addSimpleAttribute("operandValue").addSimpleAttribute(
						"ifTrueState").addSimpleAttribute("ifFalseState")
				.addSimpleAttribute("readOnly").addAttributeRef("operand")
				.setContainerProperty("displayStateHandler"));

		// gui:stateConditionJEXL
		registerSimpleParser("stateConditionJEXL", SimpleDefinitionParser
				.getInstance(ConditionalJEXLDisplayStateHandler.class)
				.addSimpleAttribute("propertyPath").addSimpleAttribute(
						"condition").addSimpleAttribute("ifTrueState")
				.addSimpleAttribute("ifFalseState").setContainerProperty(
						"displayStateHandler"));

		// gui:stateAdapted
		registerSimpleParser("stateAdapted", SimpleDefinitionParser
				.getInstance(AdaptedContextPropertyDisplayStateHandler.class)
				.addSimpleAttribute("propertyPath").addAttributeRef("adapter")
				.setContainerProperty("displayStateHandler"));

		// gui:stateAnd
		registerSimpleParser("stateAnd", SimpleDefinitionParser.getInstance(
				AndDisplayStateHandler.class).setList(null, "handlers")
				.setContainerProperty("displayStateHandler"));

		// gui:booleanStateAdapter
		registerSimpleParser("booleanStateAdapter", SimpleDefinitionParser
				.getInstance(BooleanStateAdapter.class).addSimpleAttribute(
						"ifTrueState").addSimpleAttribute("ifFalseState")
				.setContainerProperty("adapter"));

		// gui:mapStateAdapter
		registerSimpleParser("mapStateAdapter", SimpleDefinitionParser
				.getInstance(MapStateAdapter.class).addSimpleAttribute(
						"defaultState").setMap("map", "map")
				.setContainerProperty("adapter"));

		// gui:conditionalStateAdapter
		registerSimpleParser("conditionalStateAdapter", SimpleDefinitionParser
				.getInstance(ConditionalStateAdapter.class).addSimpleAttribute(
						"operator").addSimpleAttribute("ifTrueState")
				.addSimpleAttribute("operandValue", "operand").addAttributeRef(
						"operandRef").addSimpleAttribute("ifFalseState")
				.setContainerProperty("adapter"));
	}

	private void registerListDecorator(String elementName, String listName) {
		ListDecorator fieldsDecorator = new ListDecorator(listName);
		registerBeanDefinitionDecorator(elementName, fieldsDecorator);
		registerBeanDefinitionParser(elementName, fieldsDecorator);
	}

	private void registerSimpleParser(String elementName,
			SimpleDefinitionParser<?> parser) {
		registerBeanDefinitionParser(elementName, parser);
		registerBeanDefinitionDecorator(elementName, parser);
	}
}
