package net.sf.doolin.gui.validation;

import java.util.Map;

import javax.swing.JComponent;

/**
 * Utilities for the validation of components.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ValidationComponentUtils {

	private static final String MESSAGE_KEYS = "validation.messageKeys";

	private ValidationComponentUtils() {
	}

	/**
	 * Sets the field key on a component
	 * 
	 * @param component
	 *            Component to set
	 * @param messageKey
	 *            Field key to set
	 */
	public static void setMessageKey(JComponent component, Object messageKey) {
		Object[] keyArray = messageKey == null ? null
				: new Object[] { messageKey };
		setMessageKeys(component, keyArray);
	}

	/**
	 * Sets a list of field keys on a component
	 * 
	 * @param comp
	 *            Component to set
	 * @param messageKeys
	 *            Keys to set
	 */
	public static void setMessageKeys(JComponent comp, Object[] messageKeys) {
		comp.putClientProperty(MESSAGE_KEYS, messageKeys);
	}

	/**
	 * Gets the field keys for a component
	 * 
	 * @param comp
	 *            Component
	 * @return Field keys or <code>null</code> if none is defined
	 */
	public static Object[] getMessageKeys(JComponent comp) {
		return (Object[]) comp.getClientProperty(MESSAGE_KEYS);
	}

	/**
	 * Gets {@link GUIValidation} object associated with a component.
	 * 
	 * @param comp
	 *            Component
	 * @param keyMap
	 *            Cache for the component validation messages, indexed by field
	 *            key
	 * @return Validation result for this component (may be <code>null</code>)
	 */
	public static GUIValidation getAssociatedResult(JComponent comp,
			Map<Object, GUIValidation> keyMap) {
		Object[] messageKeys = getMessageKeys(comp);
		if (messageKeys == null || keyMap == null) {
			return null;
		}
		if (messageKeys.length == 1) {
			GUIValidation result = keyMap.get(messageKeys[0]);
			return result == null ? GUIValidation.EMPTY : result; // already
			// unmodifiable
		}
		GUIValidation result = null;
		for (Object element : messageKeys) {
			GUIValidation subResult = keyMap.get(element);
			if (subResult != null) {
				if (result == null) {
					result = new GUIValidation();
				}
				result.addAllFrom(subResult);
			}
		}
		return result == null ? GUIValidation.EMPTY : GUIValidation
				.unmodifiable(result);
	}

}
