package net.sf.doolin.gui.view.decorator;

import java.util.HashMap;
import java.util.Map;

/**
 * A decoration style is a set of parameters that is used by
 * {@link GUIViewDecorator}s.
 * 
 * @author Damien Coraboeuf
 */
public class DecorationStyle {

	/**
	 * Style of border. List of values are:
	 * <ul>
	 * <li>{@link #BORDER_STYLE_TITLE_BAR titleBar} (default)
	 * <li>{@link #BORDER_STYLE_NONE none}
	 * <li>{@link #BORDER_STYLE_BORDER_ONLY borderOnly}
	 * <li>{@link #BORDER_STYLE_BORDER_TITLE borderTitle}
	 * </ul>
	 */
	public static final String PARAM_BORDER_STYLE = "borderStyle";

	/**
	 * <code>titleBar</code> border style where the title is displayed in a bar
	 * at the top of the view.
	 */
	public static final String BORDER_STYLE_TITLE_BAR = "titleBar";

	/**
	 * <code>none</code> border style where no title is displayed in the view.
	 */
	public static final String BORDER_STYLE_NONE = "none";

	/**
	 * <code>borderOnly</code> border style where no title is displayed in the
	 * view but a border is drawn around the view.
	 */
	public static final String BORDER_STYLE_BORDER_ONLY = "borderOnly";

	/**
	 * <code>borderTitle</code> border style where a titled border is drawn
	 * around the view.
	 */
	public static final String BORDER_STYLE_BORDER_TITLE = "borderTitle";

	private Map<String, Object> parameterMap = new HashMap<String, Object>();

	/**
	 * Constructor
	 */
	public DecorationStyle() {
		parameterMap.put(PARAM_BORDER_STYLE, BORDER_STYLE_TITLE_BAR);
	}

	/**
	 * Gets the parameter map.
	 * 
	 * @return the parameter map
	 */
	public Map<String, Object> getParameterMap() {
		return parameterMap;
	}

	/**
	 * Sets the parameter map.
	 * 
	 * @param parameterMap
	 *            the parameter map
	 */
	public void setParameterMap(Map<String, Object> parameterMap) {
		this.parameterMap = parameterMap;
	}

	/**
	 * Gets a parameter from the map.
	 * 
	 * @param key
	 *            Key of the parameter
	 * @return Parameter value or <code>null</code> if not found.
	 */
	public Object getParamater(String key) {
		return parameterMap.get(key);
	}

	/**
	 * Tests if a parameter is defined or not.
	 * 
	 * @param key
	 *            Parameter key to test
	 * @return <code>true</code> if this decoration style defines this parameter
	 */
	public boolean isParameterDefined(String key) {
		return parameterMap.containsKey(key);
	}

}
