package net.sf.doolin.gui.window.descriptor;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;

import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.decorator.DecorationStyle;
import net.sf.doolin.gui.view.decorator.GUIViewDecorator;
import net.sf.doolin.gui.view.descriptor.GUIViewDescriptor;
import net.sf.doolin.gui.window.GUIWindow;

import org.springframework.beans.factory.BeanNameAware;

/**
 * Window descriptor that contains several views.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of data for the window
 * @param <C>
 *            Type of constraint for the views
 */
public abstract class AbstractCompositeGUIWindowDescriptor<B, C> extends
		AbstractGUIWindowDescriptor<B> implements BeanNameAware {

	private List<ViewReference<?, C>> viewReferences = new ArrayList<ViewReference<?, C>>(
			0);

	private GUIViewDecorator viewDecorator;

	/**
	 * Arrange view.
	 * 
	 * @param window
	 *            the window
	 * @param view
	 *            the view
	 * @param constraint
	 *            the constraint
	 * @param decorationStyle
	 *            the decoration style
	 * @param <V>
	 *            Type of data for the view
	 */
	protected <V> void arrangeView(GUIWindow<B> window, GUIView<V> view,
			C constraint, DecorationStyle decorationStyle) {
		// View component
		JComponent viewComponent = initViewComponent(window, view,
				this.viewDecorator, decorationStyle);
		// Arranges the view
		arrangeViewInWindow(window, view, viewComponent, constraint);
	}

	/**
	 * Arrange view.
	 * 
	 * @param window
	 *            the window
	 * @param view
	 *            the view
	 * @param viewDecoratedComponent
	 *            the view decorated component
	 * @param constraint
	 *            the constraint
	 * @param <V>
	 *            Type of data for the view
	 */
	protected abstract <V> void arrangeViewInWindow(GUIWindow<B> window,
			GUIView<V> view, JComponent viewDecoratedComponent, C constraint);

	/**
	 * Arrange view reference.
	 * 
	 * @param <V>
	 *            Type of data for the view
	 * @param viewReference
	 *            the view reference
	 * @param window
	 *            the window
	 */
	protected <V> void arrangeViewReference(ViewReference<V, C> viewReference,
			GUIWindow<B> window) {
		// Gets the view data
		V viewData = viewReference.getModelFactory().create(
				window.getWindowData());
		arrangeViewWithReference(window, viewReference, viewData);
	}

	/**
	 * Arranges the views inside the window.
	 * 
	 * @param window
	 *            Window to arrange
	 */
	@Override
	protected void arrangeViews(GUIWindow<B> window) {
		if (this.viewReferences != null) {
			for (ViewReference<?, C> viewReference : this.viewReferences) {
				// Visible at initialization time
				if (viewReference.isVisible()) {
					arrangeViewReference(viewReference, window);
				}
			}
		}
	}

	/**
	 * Arrange view with reference.
	 * 
	 * @param window
	 *            the window
	 * @param viewReference
	 *            the view reference
	 * @param viewData
	 *            the view data
	 * @param <V>
	 *            Type of data for the view
	 * @return View
	 */
	protected <V> GUIView<V> arrangeViewWithReference(GUIWindow<B> window,
			ViewReference<V, C> viewReference, V viewData) {
		// Creates the view
		GUIView<V> view = viewReference.getViewDescriptor().createView(window,
				viewData);
		// Positions the view in the window
		arrangeView(window, view, viewReference.getViewConstraint(),
				viewReference.getDecorationStyle());
		// Registers the view
		view.init();
		// OK
		return view;
	}

	/**
	 * Arranges the window.
	 * 
	 * @param window
	 *            Window
	 */
	@Override
	protected abstract void arrangeWindow(GUIWindow<B> window);

	/**
	 * Finds a view reference for the given view descriptor and view model.
	 * 
	 * @param parentWindow
	 *            Parent window
	 * @param viewDescriptor
	 *            View descriptor
	 * @param viewData
	 *            Data for the view
	 * @param <V>
	 *            Type of data for the view
	 * @return View reference
	 */
	protected <V> ViewReference<V, C> findViewReference(
			GUIWindow<B> parentWindow, GUIViewDescriptor<V> viewDescriptor,
			V viewData) {
		for (ViewReference<?, C> vr : this.viewReferences) {
			if (vr.getViewDescriptor() == viewDescriptor) {
				@SuppressWarnings("unchecked")
				ViewReference<V, C> vrOK = (ViewReference<V, C>) vr;
				return vrOK;
			}
		}
		return null;
	}

	/**
	 * Gets the view decorator.
	 * 
	 * @return the view decorator
	 */
	public GUIViewDecorator getViewDecorator() {
		return this.viewDecorator;
	}

	/**
	 * Gets the view references.
	 * 
	 * @return the view references
	 */
	public List<ViewReference<?, C>> getViewReferences() {
		return this.viewReferences;
	}

	@Override
	public <V> GUIView<V> openView(GUIWindow<B> parentWindow,
			GUIViewDescriptor<V> viewDescriptor, V viewData) {
		// View id
		String viewId = viewDescriptor.getID(viewData);
		// Gets an already open view
		GUIView<V> view = parentWindow.getView(viewId);
		if (view != null) {
			view.show();
			return view;
		} else {
			// Looks for a view reference
			ViewReference<V, C> viewReference = findViewReference(parentWindow,
					viewDescriptor, viewData);
			// No view reference
			if (viewReference == null) {
				throw new RuntimeException("Cannot find a view reference");
			}
			// Positions the new view in the window
			view = arrangeViewWithReference(parentWindow, viewReference,
					viewData);
			// Registers the view in the window
			parentWindow.addView(view);
			// OK
			return view;
		}
	}

	/**
	 * Sets the view decorator.
	 * 
	 * @param viewDecorator
	 *            the view decorator
	 */
	public void setViewDecorator(GUIViewDecorator viewDecorator) {
		this.viewDecorator = viewDecorator;
	}

	/**
	 * Sets the view references.
	 * 
	 * @param viewReferences
	 *            the new view references
	 */
	public void setViewReferences(List<ViewReference<?, C>> viewReferences) {
		this.viewReferences = viewReferences;
	}

}
