package net.sf.doolin.gui.window.descriptor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.Icon;
import javax.swing.JComponent;

import net.sf.doolin.gui.action.ExitAction;
import net.sf.doolin.gui.action.GUIAction;
import net.sf.doolin.gui.action.path.ActionBarContext;
import net.sf.doolin.gui.action.path.list.ActionPathList;
import net.sf.doolin.gui.action.path.toolbar.ToolBar;
import net.sf.doolin.gui.action.swing.ActionFactory;
import net.sf.doolin.gui.action.swing.DefaultActionFactory;
import net.sf.doolin.gui.app.GUIApplication;
import net.sf.doolin.gui.expression.GUIExpression;
import net.sf.doolin.gui.service.IconService;
import net.sf.doolin.gui.service.IconSize;
import net.sf.doolin.gui.service.icon.DefaultIconService;
import net.sf.doolin.gui.view.GUIView;
import net.sf.doolin.gui.view.decorator.DecorationStyle;
import net.sf.doolin.gui.view.decorator.GUIViewDecorator;
import net.sf.doolin.gui.window.GUIWindow;
import net.sf.doolin.gui.window.GUIWindowFactory;
import net.sf.doolin.gui.window.GUIWindowListener;
import net.sf.doolin.gui.window.support.FrameWindowFactory;
import net.sf.doolin.gui.window.support.WindowSize;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Window descriptor that provides common attributes and methods.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <B>
 *            Type of data for the window
 */
public abstract class AbstractGUIWindowDescriptor<B> implements
		GUIWindowDescriptor<B>, BeanNameAware {

	private final static Logger log = LoggerFactory
			.getLogger(AbstractGUIWindowDescriptor.class);

	private String windowId;

	private String titleExpression = "";

	private String iconId;

	private IconService iconService = new DefaultIconService();

	private ActionFactory actionFactory = DefaultActionFactory.INSTANCE;

	private WindowSize size = new WindowSize();

	private GUIAction closeAction = new ExitAction<B>();

	private GUIWindowFactory windowFactory = new FrameWindowFactory();
	private List<ToolBar> toolBars = new ArrayList<ToolBar>(0);
	private ActionPathList menuBar;

	private List<GUIWindowListener<B>> windowListeners = new ArrayList<GUIWindowListener<B>>();

	// List of opened windows
	private Map<String, GUIWindow<B>> windows = new HashMap<String, GUIWindow<B>>(
			1);

	/**
	 * Does nothing.
	 * 
	 * @param window
	 *            the window
	 * @param view
	 *            the view
	 * 
	 * @see GUIView#show()
	 */
	@Override
	public void activateView(GUIWindow<B> window, GUIView<?> view) {
		log.debug(String.format("Activate view %s", view.getID()));
	}

	/**
	 * Arranges the content of the window.
	 * 
	 * @param window
	 *            Window to arrange
	 */
	protected void arrange(final GUIWindow<B> window) {
		// General
		arrangeWindow(window);
		// Title
		final GUIExpression titleGUIExpression = new GUIExpression(window
				.getActionContext().getSubscriberValidator(), window
				.getWindowData(), this.titleExpression);
		if (titleGUIExpression.isVariable()) {
			titleGUIExpression.subscribe(window.getActionContext()
					.getSubscriberValidator(), new Runnable() {

				@Override
				public void run() {
					window.setWindowTitle(titleGUIExpression.getValue());
				}
			});
		}
		window.setWindowTitle(titleGUIExpression.getValue());
		// Icon
		if (StringUtils.isNotBlank(this.iconId)) {
			// Gets the icon
			Icon icon = this.iconService.getIcon(this.iconId, IconSize.MEDIUM);
			// Sets the icon
			window.setWindowIcon(icon);
		}
		// Bars
		arrangeActionBars(window);
		// Views
		arrangeViews(window);
		// Final set-up
		window.onPostCreate();
	}

	/**
	 * Creates and setup the action bars (menu bars, tool bars, action bars).
	 * 
	 * @param window
	 *            Window
	 */
	protected void arrangeActionBars(GUIWindow<B> window) {
		// Arrange menu bar
		if (this.menuBar != null) {
			window.installMenuBar(null, new ActionBarContext(this.menuBar,
					window.getActionContext()));
		}
		// Arrange tool bars
		if (this.toolBars != null && !this.toolBars.isEmpty()) {
			// For each tool bar
			for (ToolBar toolBar : this.toolBars) {
				window.installToolBar(toolBar, window.getActionContext());
			}
		}
	}

	/**
	 * Arranges the views inside the window.
	 * 
	 * @param window
	 *            Window to arrange
	 */
	protected abstract void arrangeViews(GUIWindow<B> window);

	/**
	 * Arranges the window.
	 * 
	 * @param window
	 *            Window
	 */
	protected abstract void arrangeWindow(GUIWindow<B> window);

	@Override
	public GUIWindow<B> createWindow(GUIApplication application, B windowData) {
		// Creates the window instance
		GUIWindow<B> window = this.windowFactory.createWindow();
		// Builds the window
		window.build(application, this, windowData);
		// Arranges the window content
		arrange(window);
		// Registers the window
		String windowId = getWindowId(windowData);
		log.info("Registering window " + windowId);
		this.windows.put(windowId, window);
		// OK
		return window;
	}

	/**
	 * Gets the action factory.
	 * 
	 * @return the action factory
	 */
	@Override
	public ActionFactory getActionFactory() {
		return this.actionFactory;
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @return the close action
	 */
	public GUIAction getCloseAction() {
		return this.closeAction;
	}

	/**
	 * Gets the icon id.
	 * 
	 * @return the icon id
	 */
	public String getIconId() {
		return this.iconId;
	}

	/**
	 * Gets the icon service.
	 * 
	 * @return the icon service
	 */
	public IconService getIconService() {
		return this.iconService;
	}

	/**
	 * Gets the {@link ActionPathList} that defines the menu bar for this
	 * window.
	 * 
	 * @return List of action paths or <code>null</code> if no menu bar is
	 *         needed
	 */
	public ActionPathList getMenuBar() {
		return this.menuBar;
	}

	/**
	 * Gets the size.
	 * 
	 * @return the size
	 */
	public WindowSize getSize() {
		return this.size;
	}

	/**
	 * Gets the title expression.
	 * 
	 * @return the title expression
	 */
	public String getTitleExpression() {
		return this.titleExpression;
	}

	/**
	 * Returns the list of tool bars for this window.
	 * 
	 * @return List of tool bars
	 */
	public List<ToolBar> getToolBars() {
		return this.toolBars;
	}

	@Override
	public GUIWindow<B> getWindow(GUIApplication application, B windowData) {
		// Gets the window ID
		String windowId = getWindowId(windowData);
		// Returns the window
		return this.windows.get(windowId);
	}

	/**
	 * Gets the id.
	 * 
	 * @return the id
	 */
	public String getWindowId() {
		return this.windowId;
	}

	/**
	 * Gets the ID of a window model.
	 * 
	 * @param windowData
	 *            Window model
	 * 
	 * @return ID
	 */
	@Override
	public String getWindowId(B windowData) {
		GUIExpression expression = new GUIExpression(null, windowData,
				this.windowId);
		return expression.getValue();
	}

	public List<GUIWindowListener<B>> getWindowListeners() {
		return this.windowListeners;
	}

	/**
	 * Does nothing.
	 * 
	 * @param window
	 *            the window
	 * @param view
	 *            the view
	 */
	@Override
	public void hideView(GUIWindow<B> window, GUIView<?> view) {
		log.debug(String.format("Hidding view %s", view.getID()));
	}

	/**
	 * Initialises a view before its usage in this window.
	 * 
	 * @param <V>
	 *            Type of bean for the view
	 * @param window
	 *            Hosting window
	 * @param view
	 *            View to initialise
	 * @param additionalViewDecorator
	 *            Decorator for the view
	 * @param decorationStyle
	 *            Style for the decoration
	 * @return Decorated view component for the window
	 */
	protected <V> JComponent initViewComponent(GUIWindow<B> window,
			GUIView<V> view, GUIViewDecorator additionalViewDecorator,
			DecorationStyle decorationStyle) {
		// View component
		JComponent viewComponent = view.getViewComponent();
		// Decorator for the view ?
		GUIViewDecorator specificDecorator = view.getViewDescriptor()
				.getViewDecorator();
		if (specificDecorator != null) {
			viewComponent = specificDecorator.decorate(view, viewComponent,
					decorationStyle);
		}
		// Decorates the view
		if (additionalViewDecorator != null) {
			viewComponent = additionalViewDecorator.decorate(view,
					viewComponent, decorationStyle);
		}
		// Initialization
		view.getViewDescriptor().initState(view);
		// OK
		return viewComponent;
	}

	@Override
	public void onCloseWindow(GUIWindow<B> window) {
		// Gets the window ID
		String windowId = window.getId();
		// Unregisters the window
		log.info("Unregistering window " + windowId);
		this.windows.remove(windowId);
	}

	/**
	 * Does nothing y default
	 */
	@Override
	public void onClosingWindow(GUIWindow<B> window) {
	}

	/**
	 * Does nothing
	 */
	@Override
	public void onOpenedWindow(GUIWindow<B> window) {
	}

	/**
	 * Sets the action factory.
	 * 
	 * @param actionFactory
	 *            the new action factory
	 */
	@Autowired(required = false)
	public void setActionFactory(ActionFactory actionFactory) {
		this.actionFactory = actionFactory;
	}

	@Override
	public void setBeanName(String name) {
		if (this.windowId == null) {
			this.windowId = name;
		}
	}

	/**
	 * Sets the close action.
	 * 
	 * @param closeAction
	 *            the new close action
	 */
	public void setCloseAction(GUIAction closeAction) {
		this.closeAction = closeAction;
	}

	/**
	 * Sets the icon id.
	 * 
	 * @param iconId
	 *            the new icon id
	 */
	public void setIconId(String iconId) {
		this.iconId = iconId;
	}

	/**
	 * Sets the icon service.
	 * 
	 * @param iconService
	 *            the new icon service
	 */
	@Autowired(required = false)
	public void setIconService(IconService iconService) {
		this.iconService = iconService;
	}

	/**
	 * Sets the {@link ActionPathList} that defines the menu bar for this
	 * window. It is set to <code>null</code> by default.
	 * 
	 * @param menuBar
	 *            List of action paths
	 */
	public void setMenuBar(ActionPathList menuBar) {
		this.menuBar = menuBar;
	}

	/**
	 * Sets the size.
	 * 
	 * @param size
	 *            the new size
	 */
	public void setSize(WindowSize size) {
		this.size = size;
	}

	/**
	 * Sets the title expression.
	 * 
	 * @param titleExpression
	 *            the new title expression
	 */
	public void setTitleExpression(String titleExpression) {
		this.titleExpression = titleExpression;
	}

	/**
	 * Sets the list of tool bars for this window
	 * 
	 * @param toolBars
	 *            List of tool bars
	 */
	public void setToolBars(List<ToolBar> toolBars) {
		this.toolBars = toolBars;
	}

	/**
	 * Sets the factory for the window implementation. It is set by default to
	 * {@link FrameWindowFactory}.
	 * 
	 * @param windowFactory
	 *            Factory for the window implementation
	 */
	public void setWindowFactory(GUIWindowFactory windowFactory) {
		this.windowFactory = windowFactory;
	}

	/**
	 * Sets the id.
	 * 
	 * @param id
	 *            the new id
	 */
	public void setWindowId(String id) {
		this.windowId = id;
	}

	public void setWindowListeners(List<GUIWindowListener<B>> windowListeners) {
		this.windowListeners = windowListeners;
	}

}
