package net.sf.doolin.gui.docking.storage;

import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import net.sf.doolin.gui.docking.DockDescription;
import net.sf.doolin.gui.window.GUIWindow;
import net.sf.doolin.oxml.OXMLReader;
import net.sf.doolin.oxml.config.XMLOXMLConfig;
import net.sf.doolin.oxml.source.StreamOXMLSource;
import net.sf.doolin.template.Template;
import net.sf.doolin.template.TemplateEngine;
import net.sf.doolin.template.TemplateManager;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * Implementation of {@link DockingStorage} that stores the docking description
 * as XML, using a {@link Storage} instance for the actual storage.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class XMLDockingStorage implements DockingStorage {

	private static final Logger log = LoggerFactory.getLogger(XMLDockingStorage.class);

	private TemplateManager templateManager;
	private String templatePath = "/net/sf/doolin/gui/docking/DockingTemplate.xml";
	private String oxmlPath = "/net/sf/doolin/gui/docking/DockingOXML.xml";

	private Storage storage = new DefaultStorage();

	@Override
	public DockDescription retrieve(GUIWindow<?> window) {
		// Window id
		String windowId = window.getId();
		try {
			// Retrieves the XML content
			String xml = this.storage.retrieve(window);
			if (StringUtils.isBlank(xml)) {
				return null;
			}
			// OXML configuration
			URL oxmlURL = XMLDockingStorage.class.getResource(this.oxmlPath);
			XMLOXMLConfig config = new XMLOXMLConfig(oxmlURL);
			// OXML reader
			OXMLReader<OXMLDocking> reader = new OXMLReader<OXMLDocking>(config);
			// XML input
			BufferedInputStream in = new BufferedInputStream(
					new ByteArrayInputStream(xml.getBytes("UTF-8")));
			try {
				// XML source
				StreamOXMLSource source = new StreamOXMLSource(in);
				// Reads the XML
				OXMLDocking o = reader.read(source);
				// Creates the description
				DockDescription description = o.convert();
				// OK
				return description;
			} finally {
				in.close();
			}
		} catch (IOException ex) {
			log.error(String.format(
					"Cannot retrieve docking description for window %s",
					windowId), ex);
			return null;
		}
	}

	/**
	 * Sets the {@link Storage} that actually stores the XML for the window
	 * 
	 * @param storage
	 *            Storage
	 */
	public void setStorage(Storage storage) {
		this.storage = storage;
	}

	/**
	 * Sets the template manager which is used for the XML generation.
	 * 
	 * @param templateManager
	 *            Template manager
	 */
	@Autowired
	public void setTemplateManager(TemplateManager templateManager) {
		this.templateManager = templateManager;
	}

	@Override
	public void store(GUIWindow<?> window, DockDescription description) {
		// Gets the template
		TemplateEngine templateEngine = this.templateManager
				.getTemplateEngine(TemplateEngine.VELOCITY);
		Template template = templateEngine.getTemplate(this.templatePath);
		// Context
		Map<String, Object> context = new HashMap<String, Object>();
		context.put("description", description);
		context.put("formatter", new DockingFormatter());
		// Output
		StringWriter sw = new StringWriter();
		PrintWriter out = new PrintWriter(new BufferedWriter(sw));
		try {
			// Generates the template
			template.generate(out, context);
		} finally {
			out.close();
		}
		// Gets the XML content
		String xml = sw.toString();
		// Stores the XML content
		this.storage.store(window, xml);
	}

}
